# quick script to produce a DOT file of the steps from a list of dosleg
# use "python steps_as_dot.py <data_directory> | dot -Tpng > steps.png" to produce the diagram

# the XKCD font is available here: https://github.com/ipython/xkcd-font/tree/master/xkcd/build
import sys, os, random, glob

from tlfp.tools.common import open_json, print_json

if len(sys.argv) < 2:
    print('USAGE: "python steps_as_dot.py <data_directory> | dot -Tpng > steps.png"')
    sys.exit()

mode = "detailed" if len(sys.argv) == 3 else "simple"

procedure_file = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), '..', 'docs', 'valid_procedure.json')
procedure = open_json(procedure_file)

API_DIRECTORY = sys.argv[1]
all_senat_jo = [open_json(path) for path \
                in glob.glob(os.path.join(API_DIRECTORY, '*/viz/procedure.json'))]
all_senat_jo = [dos for dos in all_senat_jo if dos.get('end')]
# all_senat_jo = [x for x in open_json(sys.argv[1]) if len(x['steps']) > 2]
# all_senat_jo = random.sample(all_senat_jo, 5)

nodes_names_size = {}
step_trans = {}
steps_logs = ""
for dos in all_senat_jo:
    prev_step = None
    last_step = ''
    for step_i, step in enumerate(dos.get('steps', [])):
        if mode == "simple":
            step_name = ' • '.join((x for x in (step.get('stage'), step.get('institution')) if x))
            if "CMP" in step_name:
                step_name = "CMP"
        else:
            step_name = ' • '.join((x for x in (step.get('stage'), step.get('institution'), step.get('step')) if x))
        if step_name:
            if mode == "detailed" or not (prev_step and prev_step.get('step') == 'depot' and step.get('step') == 'depot'):
                if last_step not in step_trans:
                    step_trans[last_step] = {}
                step_trans[last_step][step_name] = step_trans[last_step].get(step_name, 0) + 1
                nodes_names_size[step_name] = nodes_names_size.get(step_name, set()).union(set([dos.get('url_dossier_senat')]))
                steps_logs += '%s->%s:%s\n' % (last_step, step_name, dos.get('url_dossier_assemblee'))
            last_step = step_name
            prev_step = step

dot_result = """digraph g {
    node  [style="rounded,filled,bold", shape=box, fontname="xkcd"];
    edge  [style=bold, fontname="xkcd"];
    ranksep=0;
"""

nodes_names_i = 0
nodes_names = {}
def get_node_id(node):
    global nodes_names_i, nodes_names
    if node not in nodes_names:
        nodes_names[node] = nodes_names_i
        nodes_names_i += 1
    return nodes_names[node]

for prev, nexts in step_trans.items():
    if prev:
        prev_id = get_node_id(prev)
        for next, next_v in nexts.items():
            next_id = get_node_id(next)
            if next_id == prev_id and mode == "simple": continue

            incorrect = mode == "detailed" and procedure.get(prev, {}).get(next, False) is False
            color = '#F44336' if incorrect else '#a5a5a5'

            dot_result += '\n   %s -> %s [label="%s", penwidth="%d", color="%s", fontcolor="%s"];' % (
                prev_id, next_id, next_v, next_v // 100 + 1, color, color)

def xpos(n):
    if "senat" in n:
        return 0
    if "assemblee" in n:
        return 2
    return 1

def ypos(n):
    if "promulgation" in n:
        return 0
    if "constitutionnalité" in n:
        return 1
    if "définitive" in n:
        if "hemicycle" in n:
            return 2
        return 3
    if "nouv." in n:
        res = 4
    elif "3ème" in n:
        res = 7
    elif "2ème" in n:
        res = 10
    elif "1ère" in n:
        res = 13
    elif "CMP" in n:
        res = 8.5
    if "commission" in n:
        return res + 1
    if "depot" in n:
        return res + 2
    return res

def clean(n):
    if "assemblee" in n:
        n = n.replace("assemblee", "AN")
    if "senat" in n:
        n = n.replace("senat", "Sénat")
    if "nouv. lect" in n:
        n = n.replace("nouv. lect.", "Nouvelle lecture")
    if "définitive" in n:
        n = n.replace("l. définitive", "Lecture définitive")
    if n == "CMP • CMP":
        return "CMP • commission"
    if n == "constitutionnalité • conseil constitutionnel":
        return "Conseil Constitutionnel"
    if "promulgation" in n:
        return "Promulgation JO"
    if "congrès" in n:
        return "Congrès"
    return n

for name, id in nodes_names.items():
    # add previous step, mockup
    # prev_id = id-1 if id > 0 else id + 1
    # dot_result += '\n   %s -> %s [label="prec", penwidth="1", color="green", fontcolor="green"];' % (
    #    id, prev_id)

    # generate node
    fillcolor = "#f3f3f3"
    if 'assemblee' in name:
        fillcolor = '#ced6ff6d'
    if 'senat' in name:
        fillcolor = '#f99b906d'
    if 'CMP' in name:
        fillcolor = '#e7dd9e6d'
    if 'constitutionnalité' in name:
        fillcolor = '#aeeaaa6d'
    if 'congrès' in name:
        fillcolor = '#dfb3f36d'
    dot_result += '\n %s [label="%s • %s", penwidth="%d", fillcolor="%s"];' % (
        id,
        clean(name),
        len(nodes_names_size[name]),
        len(nodes_names_size[name]) // 600 + 1,
        fillcolor)

depot = " • depot" if mode == "detailed" else ""
if '1ère lecture • assemblee' + depot in nodes_names:
    dot_result += ("""
      {
        rank=source; %s; %s;
      }
    """ % (get_node_id('1ère lecture • assemblee' + depot), get_node_id('1ère lecture • senat' + depot)))


if mode == "simple":
    for stage in ['1ère lecture', '2ème lecture']:
        dot_result += ("""
          {
            rank=same; %s; %s;
          }
        """ % (get_node_id('%s • assemblee' % stage), get_node_id('%s • senat' % stage)))
    dot_result += """
    {
        rank=same; %s; %s; %s;
    }
    """ % (get_node_id('CMP'), get_node_id('3ème lecture • assemblee'), get_node_id('3ème lecture • senat'))

    dot_result += """
    {
        rank=same; %s; %s;
    }
    """ % (get_node_id('congrès • congrès'), get_node_id('constitutionnalité • conseil constitutionnel'))
else:
    for stage in ['1ère lecture', '2ème lecture', '3ème lecture', 'CMP']:
        for step in ['depot', 'commission', 'hemicycle']:
            if stage == 'CMP' and step == 'commission': continue
            dot_result += ("""
              {
                rank=same; %s; %s;
              }
            """ % (get_node_id('%s • assemblee • %s' % (stage, step)), get_node_id('%s • senat • %s' % (stage, step))))


dot_result += '\n}'

details = "_detailed" if mode == "detailed" else ""
open('_steps%s.log' % details, 'w').write(steps_logs)
filename = os.path.join(API_DIRECTORY, 'stats',
                        'steps%s_transitions.json' % details)
print_json(step_trans, filename)

print(dot_result)
# open('steps.dot','w').write(dot_result)


# improve layout: https://stackoverflow.com/questions/11588667/how-to-influence-layout-of-graph-items



import glob, os, sys, csv, traceback

from lawfactory_utils.urls import enable_requests_cache
from senapy.dosleg import opendata
from senapy.dosleg.parser import guess_legislature

from tlfp.parse_one import *
from tlfp.tools import parse_texte
from tlfp.tools.common import upper_first, format_date, datize, strip_text, open_json
from tlfp.tools.process_conscons import get_decision_length
from tlfp.tools.process_jo import count_signataires, get_texte_length
from tlfp.tools.update_procedure import detect_auteur_depot_from_url


def annee(date):
    return int(date.split('/')[-1])


def find_last_depot(steps):
    last_depot = None
    for step in steps:
        if not step.get('step') == 'depot':
            break
        last_depot = step
    return last_depot


def parse_senat_open_data(run_old=False):
    senat_csv = opendata.fetch_csv()
    senat_csv = [dos for dos in senat_csv if dos['Date initiale']]

    # filter before 2008
    if run_old:
        senat_csv = [dos for dos in senat_csv if annee(dos['Date initiale']) < 2008]
    else:
        senat_csv = [dos for dos in senat_csv if annee(dos['Date initiale']) >= 2008]
    senat_csv.sort(key=lambda dos: annee(dos['Date initiale']))
    return senat_csv


def find_parsed_doslegs(api_directory):
    dossiers_json = {}
    for path in glob.glob(os.path.join(api_directory, '**/procedure.json'), recursive=True):
        dos = open_json(path)
        if dos.get('senat_id'):
            dossiers_json[dos['senat_id']] = dos
    print(len(dossiers_json), 'parsed found')
    return dossiers_json


def count_echecs(steps):
    return len([s for s in steps if s.get('echec')])


def get_CMP_type(steps):
    steps = [s for s in steps if s.get('stage') == 'CMP']
    if not steps:
        return 'pas de CMP'
    if len(steps) == 3 and not any([s.get('echec') for s in steps]):
        return 'succès'
    return 'échec'


def get_initiative(steps):
    real_depot = None
    for step in steps:
        if step.get('step') != 'depot':
            break
        real_depot = step
    return detect_auteur_depot_from_url(real_depot.get('source_url', '')), real_depot.get('institution')


def read_text(articles):
    texte = []
    for art in articles:
        if 'alineas' in art:
            for key in sorted(art['alineas'].keys()):
                if art['alineas'][key] != '':
                    texte.append(strip_text(art['alineas'][key]))
    return len("\n".join(texte))


def add_metrics(dos, parsed_dos, fast=False):
    parsed_dos = dossiers_json[senat_id]
    stats = parsed_dos['stats']
    dos['Titre court'] = parsed_dos['short_title']
    dos["URL du dossier Assemblée"] = parsed_dos.get('url_dossier_assemblee', '')
    dos['Type de procédure'] = "accélérée" if parsed_dos['urgence'] else "normale"
    dos['Initiative du texte'], dos['Institution de dépôt'] = get_initiative(parsed_dos['steps'])
    dos['Étapes échouées'] = count_echecs(parsed_dos['steps'])
    dos['CMP'] = get_CMP_type(parsed_dos['steps'])
    cc_step = [step['source_url'] for step in parsed_dos['steps'] if step.get('stage') == 'constitutionnalité']
    dos['URL CC'] = cc_step[0] if cc_step else ''
    dos["Nombre d'articles censurés"] = stats.get('censored_articles', 0)
    dos["Nombre d'articles totalement censurés"] = stats.get('fully_censored_articles', 0)
    dos["Législature de promulgation"] = parsed_dos['assemblee_legislature']
    dos["Nombre de propositions de loi rattachées"] = stats['attached_law_proposals']
    dos["Nombre de textes produits"] = stats['texts_produced']
    dos["Nombre de dépôts dans les institutions"] = stats['depots_in_institutions']

    """
    if not fast:
        dos['Taille de la décision du CC'] = get_decision_length(cc_step[0]) if cc_step else ''
        dos['Signataires au JO'] = count_signataires(parsed_dos['url_jo']) if 'url_jo' in parsed_dos else ''
    """
    dos['URL JO'] = parsed_dos['url_jo'] if 'url_jo' in parsed_dos else ''

    dos['Nombre de caractères final'] = stats['output_text_length']
    dos["Nombre de caractères avant saisine"] = stats.get('output_text_before_CC_length', stats['output_text_length'])
    dos['Nombre de caractères initial'] = stats['input_text_length']
    dos["Croissance du nombre de caractères"] = stats['ratio_text_length_growth']
    dos['Proportion de texte modifié'] = stats['ratio_texte_modif']
    dos["Nombre d'articles initial"] = stats['total_input_articles']
    dos["Nombre d'articles final"] = stats['total_output_articles']
    dos["Nombre d'articles avant saisine"] = dos["Nombre d'articles final"] + dos["Nombre d'articles totalement censurés"]
    dos["Croissance du nombre d'articles"] = stats['ratio_articles_growth']
    dos["Nombre de mots initial"] = stats['input_text_word_count']
    dos["Nombre de mots avant saisine"] = stats.get('output_text_before_CC_word_count', stats['output_text_word_count'])
    dos["Nombre de mots final"] = stats['output_text_word_count']
    dos["Croissance du nombre de mots"] = stats['ratio_word_count_growth']
    dos["Nombre de mots prononcés"] = stats['total_mots']
    dos["Nombre d'intervenants"] = stats['total_intervenants']
    dos["Nombre d'interventions"] = stats['total_interventions']
    dos["Nombre de séances"] = stats['total_seances']
    dos["Nombre de séances à l'Assemblée"] = stats['total_seances_assemblee']
    dos["Nombre de séances au Sénat"] = stats['total_seances_senat']
    dos["Dernière lecture"] = stats['last_stage']
    dos["Dernière institution"] = stats['last_institution']

    dos["Nombre d'amendements"] = stats['total_amendements']
    dos["Nombre d'amendements adoptés"] = stats['total_amendements_adoptes']
    dos["Nombre d'amendements du Gouvernement"] = stats['total_amendements_gouvernement']
    dos["Nombre d'amendements du Gouvernement adoptés"] = stats['total_amendements_gouvernement_adoptes']
    dos["Nombre d'amendements des sénateurs"] = stats['total_amendements_senateurs']
    dos["Nombre d'amendements des sénateurs adoptés"] = stats['total_amendements_senateurs_adoptes']
    dos["Nombre d'amendements du Gouvernement au Sénat"] = stats['total_amendements_gouvernement_senat']
    dos["Nombre d'amendements du Gouvernement au Sénat adoptés"] = stats['total_amendements_gouvernement_senat_adoptes']
    dos["Nombre d'amendements des députés"] = stats['total_amendements_deputes']
    dos["Nombre d'amendements des députés adoptés"] = stats['total_amendements_deputes_adoptes']
    dos["Nombre d'amendements du Gouvernement à l'Assemblée"] = stats['total_amendements_gouvernement_assemblee']
    dos["Nombre d'amendements du Gouvernement à l'Assemblée adoptés"] = stats['total_amendements_gouvernement_assemblee_adoptes']

    dos["Nombre d'amendements en hémicycle"] = stats['total_amendements_hemicycle']
    dos["Nombre d'amendements en hémicycle adoptés"] = stats['total_amendements_hemicycle_adoptes']
    dos["Nombre d'amendements en hémicycle du Gouvernement"] = stats['total_amendements_hemicycle_gouvernement']
    dos["Nombre d'amendements en hémicycle du Gouvernement adoptés"] = stats['total_amendements_hemicycle_gouvernement_adoptes']
    dos["Nombre d'amendements en hémicycle des sénateurs"] = stats['total_amendements_hemicycle_senateurs']
    dos["Nombre d'amendements en hémicycle des sénateurs adoptés"] = stats['total_amendements_hemicycle_senateurs_adoptes']
    dos["Nombre d'amendements en hémicycle du Gouvernement au Sénat"] = stats['total_amendements_hemicycle_gouvernement_senat']
    dos["Nombre d'amendements en hémicycle du Gouvernement au Sénat adoptés"] = stats['total_amendements_hemicycle_gouvernement_senat_adoptes']
    dos["Nombre d'amendements en hémicycle des députés"] = stats['total_amendements_hemicycle_deputes']
    dos["Nombre d'amendements en hémicycle des députés adoptés"] = stats['total_amendements_hemicycle_deputes_adoptes']
    dos["Nombre d'amendements en hémicycle du Gouvernement à l'Assemblée"] = stats['total_amendements_hemicycle_gouvernement_assemblee']
    dos["Nombre d'amendements en hémicycle du Gouvernement à l'Assemblée adoptés"] = stats['total_amendements_hemicycle_gouvernement_assemblee_adoptes']

    dos['Textes cités'] = '|'.join(parsed_dos['textes_cites'])
    dos['Nombre de textes cités'] = len(parsed_dos['textes_cites'])

    dos['URL OpenData La Fabrique'] = 'https://www.lafabriquedelaloi.fr/api/%s/' % parsed_dos['id']


def add_metrics_via_adhoc_parsing(dos, log=sys.stderr):
    senat_dos = download_senat(dos['URL du dossier'], log=log)
    if not senat_dos:
        print('  /!\ INVALID SENAT DOS')
        return

    # Add AN version if there's one
    parsed_dos = senat_dos
    if 'url_dossier_assemblee' in senat_dos:
        dos['URL du dossier Assemblée'] = senat_dos.get('url_dossier_assemblee')
        an_dos = download_an(senat_dos['url_dossier_assemblee'], senat_dos['url_dossier_senat'], log=log)
        if 'url_dossier_senat' in an_dos and are_same_doslegs(senat_dos, an_dos):
            parsed_dos = merge_senat_with_an(senat_dos, an_dos)
    dos['Titre court'] = parsed_dos['short_title']
    dos['Type de procédure'] = "accélérée" if parsed_dos['urgence'] else "normale"
    dos['Initiative du texte'], dos['Institution de dépôt'] = get_initiative(parsed_dos['steps'])
    dos['Étapes échouées'] = count_echecs(parsed_dos['steps'])
    dos['CMP'] = get_CMP_type(parsed_dos['steps'])
    cc_step = [step['source_url'] for step in parsed_dos['steps'] if step.get('stage') == 'constitutionnalité']
    # dos['Taille de la décision du CC'] = get_decision_length(cc_step[0]) if cc_step else ''
    dos['URL CC'] = cc_step[0] if cc_step else ''
    # dos['Signataires au JO'] = count_signataires(parsed_dos['url_jo']) if 'url_jo' in parsed_dos else ''
    dos['URL JO'] = parsed_dos['url_jo'] if 'url_jo' in parsed_dos else ''
    dos["Législature de promulgation"] = parsed_dos.get('assemblee_legislature', '')

    last_depot = find_last_depot(parsed_dos['steps'])
    last_text = None
    for step in reversed(parsed_dos['steps']):
        last_text = step
        if step.get('step') == 'hemicycle':
            break
        if step.get('step') == 'commission':
            raise Exception('commission as last step')

    if 'source_url' in last_text:
        try:
            articles = parse_texte.parse(last_text['source_url'])
            if articles and articles[0].get('definitif'):
                dos['Nombre de caractères final'] = read_text(parse_texte.parse(last_text['source_url']))
            else:
                dos['Nombre de caractères final'] = get_texte_length(parsed_dos['url_jo']) if 'url_jo' in parsed_dos else ''
        except:
            print("WARNING: Nombre de caractères final impossible to evaluate")

    try:
        input_text_length = read_text(parse_texte.parse(last_depot['source_url']))
        if input_text_length > 0:
            dos['Nombre de caractères initial'] = input_text_length
    except:
        print("WARNING: Nombre de caractères initial impossible to evaluate")

    # TODO
    # dos['Proportion de texte modifié'] = ...
    # dos["Nombre d'articles initial"] = ...
    # dos["Nombre d'articles final"] = ...
    # dos["Croissance du nombre d'articles"] = ...


def clean_type_dossier(dos):
#   TODO ?  propositions de résolution ? (attention : statut = adopté pas promulgué)
    typ = dos['Type de dossier'].lower()
    for t in ['constitutionnel', 'organique']:
        if t in typ:
            return t
    for t in ['finance', 'règlement']:
        if t in typ:
            return 'budgétaire'
    tit = dos['Titre'].lower()
    # lois de programmation budgétaire seem to follow regular procédures, so set as programmation rather than budgétaire
    for t in ['programmation', 'loi de programme']:
        if t in tit:
            return 'programmation'
    for t in ['financement de la sécurité', 'règlement des comptes']:
        if t in tit:
            return 'budgétaire'
    if 'accord international' not in tit:
        if (' ratifi' in tit and 'ordonnance' in tit.split(' ratifi')[1]):
            return "ratification d'ordonnances"
        for t in [
            "autorisant le Gouvernement",
            "habilitant le gouvernement",
            "habilitation du Gouvernement",
            "habilitation à prendre par ordonnance",
            "loi d'habilitation",
            "transposition par ordonnances"
        ]:
            if t in tit:
                return "habilitation d'ordonnances"
    if typ.startswith('projet'):
        for t in ['autorisa', 'approba', 'ratifi', ' accord ', 'amendement', 'convention']:
            if t in tit:
                tit2 = " ".join(tit.split(t)[1:])
                for d in ['accord', 'avenant', 'adhésion', 'traité', 'france',
                  'gouvernement français', 'gouvernement d', 'coopération',
                  'protocole', 'arrangement', 'approbation de la décision',
                  'convention', 'ratification de la décision', 'principauté',
                  'international']:
                    if d in tit2:
                        return 'accord international'
    #for t in ["dispositions d'adaptation", 'transposition de la directive', 'portant adaptation']:
    #    if t in tit:
    #        return 'transposition EU'
    return 'ordinaire'


HEADERS = [
    "Numéro de la loi",
#   "IDs internes/institutions"
    "Titre",
    "Titre court",
    "Date initiale",
    "Année initiale",
    "Législature initiale",
    "Date de promulgation",
    "Année de promulgation",
    "Législature de promulgation",
    "Durée d'adoption (jours)",
    "Initiative du texte",
    "Institution de dépôt",
    "Nombre de propositions de loi rattachées",
    "Nombre de textes produits",
    "Nombre de dépôts dans les institutions",
    "Nombre de caractères initial",
    "Nombre de caractères avant saisine",
    "Nombre de caractères final",
    "Croissance du nombre de caractères",
#   "Proportion de texte allongé"
    "Proportion de texte modifié",
    "Nombre d'articles initial",
    "Nombre d'articles avant saisine",
    "Nombre d'articles final",
    "Croissance du nombre d'articles",
    "Nombre de mots initial",
    "Nombre de mots avant saisine",
    "Nombre de mots final",
    "Croissance du nombre de mots",
#   "Nombre d'articles inchangés",
#   "Proportion d'articles inchangés",
#   "Nombre initial d'alinéas",
#   "Nombre final d'alinéas",

    "Nombre d'amendements",
    "Nombre d'amendements adoptés",
    "Nombre d'amendements du Gouvernement",
    "Nombre d'amendements du Gouvernement adoptés",
    "Nombre d'amendements des sénateurs",
    "Nombre d'amendements des sénateurs adoptés",
    "Nombre d'amendements du Gouvernement au Sénat",
    "Nombre d'amendements du Gouvernement au Sénat adoptés",
    "Nombre d'amendements des députés",
    "Nombre d'amendements des députés adoptés",
    "Nombre d'amendements du Gouvernement à l'Assemblée",
    "Nombre d'amendements du Gouvernement à l'Assemblée adoptés",

    "Nombre d'amendements en hémicycle",
    "Nombre d'amendements en hémicycle adoptés",
    "Nombre d'amendements en hémicycle du Gouvernement",
    "Nombre d'amendements en hémicycle du Gouvernement adoptés",
    "Nombre d'amendements en hémicycle des sénateurs",
    "Nombre d'amendements en hémicycle des sénateurs adoptés",
    "Nombre d'amendements en hémicycle du Gouvernement au Sénat",
    "Nombre d'amendements en hémicycle du Gouvernement au Sénat adoptés",
    "Nombre d'amendements en hémicycle des députés",
    "Nombre d'amendements en hémicycle des députés adoptés",
    "Nombre d'amendements en hémicycle du Gouvernement à l'Assemblée",
    "Nombre d'amendements en hémicycle du Gouvernement à l'Assemblée adoptés",

    # warning: qualité moyenne des interventions
    "Nombre de mots prononcés", # (+ ventilation Gouv/AN/Sénat)
    "Nombre d'interventions",
    "Nombre d'intervenants",
    "Nombre de séances",
    "Nombre de séances à l'Assemblée",
    "Nombre de séances au Sénat",

    "Dernière lecture", # exemple d'application: si lecture déf., alors il y a beaucoup de chance que le Sénat se soit fait écrasé/bypassé
    "Dernière institution",

    # institution de dépôt pour étudier la corrélation du succés d'une CMP si le dépot commence au Sénat ou à l'AN
    # on voit déja que toutes les PPL déposées au Sénat en accéléré on réussi leur CMP

    "Type de texte",
    "Type de procédure",
    "Étapes échouées",
    "CMP",
    "Décision du CC",
    "Date de la décision du CC",
    # "Taille de la décision du CC",
    "Nombre d'articles censurés",
    "Nombre d'articles totalement censurés",
    "Textes cités",
    "Nombre de textes cités",
    # "Signataires au JO",
    "Thèmes",
    "URL du dossier Sénat",
    "URL du dossier Assemblée",
    "URL JO",
    "URL CC",
    "URL OpenData La Fabrique",
    "Texte manquant",
]


# TODO:
# - check accords internationaux : taille should include annexes and finale == initiale

if __name__ == '__main__':
    verbose = "--quiet" not in sys.argv
    fast_mode = "--fast" in sys.argv # no network requests
    if not verbose:
        sys.argv.remove("--quiet")
    args = [arg for arg in sys.argv[1:] if '--' not in arg]
    run_old = len(args) > 1

    senat_csv = parse_senat_open_data(run_old=run_old)

    enable_requests_cache()

    dossiers_json = find_parsed_doslegs(args[0])

    # silence the output if we use the --quiet flag
    with log_print(only_log=not verbose) as log:
        c = 0
        fixed = 0
        for dos in senat_csv:
            if verbose:
                print()
                print(dos['URL du dossier'])

            senat_id = dos['URL du dossier'].split('/')[-1].replace('.html', '')

            # sometimes the Senate forget to mark the dosleg as promulgated, let's fix it
            if not dos['Date de promulgation']:
                if senat_id in dossiers_json and dossiers_json[senat_id].get('url_jo'):
                    parsed_dos = dossiers_json[senat_id]
                    yy, mm, dd = parsed_dos.get('end').split('-')
                    dos['Date de promulgation'] = '%s/%s/%s' % (dd, mm, yy)
                    dos['État du dossier'] = 'promulgué'
                    # dos['Numéro de la loi'] # TODO: law_name not yet parsed in anpy
                else:
                    continue

            dos['Année initiale'] = annee(dos['Date initiale'])
            dos['Date initiale'] = format_date(dos['Date initiale'])
            dos['Législature initiale'] = guess_legislature(dos['Date initiale'])
            dos['Année de promulgation'] = annee(dos['Date de promulgation'])
            dos['Date de promulgation'] = format_date(dos['Date de promulgation'])
            dos["Durée d'adoption (jours)"] = (datize(dos["Date de promulgation"]) - datize(dos["Date initiale"])).days + 1

            dos["URL du dossier Sénat"] = dos["URL du dossier"]
            dos["Thèmes"] = dos["Thèmes"].replace(', ', '|')

            dos['Type de texte'] = clean_type_dossier(dos)

            dos['Texte manquant'] = 'oui'

            if senat_id in dossiers_json:
                if verbose:
                    print(' - matched')
                c += 1
                parsed_dos = dossiers_json[senat_id]
                try:
                    add_metrics(dos, parsed_dos, fast=fast_mode)
                    dos['Texte manquant'] = 'non'
                except KeyboardInterrupt:
                    break
            elif not fast_mode:
                # do a custom parsing when the parsed dos is missing
                try:
                    add_metrics_via_adhoc_parsing(dos, log=log)
                    fixed += 1
                except KeyboardInterrupt:
                    break
                except Exception as e:
                    traceback.print_tb(e.__traceback__)
                    print('- adhoc parsing failed for', dos['URL du dossier'], e)
                    continue

            if not dos["Décision du CC"]:
                if dos.get("URL CC"):
                    dos["Décision du CC"] = "conforme"
                else:
                    dos["Décision du CC"] = "pas de saisine"
            elif not dos.get("URL CC"):
                dos["Décision du CC"] = "pas de saisine"
            dos["Date de la décision du CC"] = format_date(dos["Date de la décision"])

            """
            if dos.get('Taille de la décision du CC') == -1:
                dos['Taille de la décision du CC'] = ''
            if dos.get('Signataires au JO') == -1:
                dos['Signataires au JO'] = ''
            """

    # filter non-promulgués after the data has been fixed
    senat_csv = [dos for dos in senat_csv if dos['Date de promulgation']]

    print(len(senat_csv), 'dos')
    print(c, 'matched')
    print(fixed, 'fixed')
    senat_csv.sort(key=lambda x: x['Date de promulgation'])

    # output the metrics CSV
    statsdir = os.path.join(sys.argv[1], 'stats')
    os.makedirs(statsdir, exist_ok=True)
    out = os.path.join(statsdir, 'metrics%s.csv' % ('-old' if run_old else ''))
    print('output:', out)
    writer = csv.DictWriter(open(out, 'w'), fieldnames=HEADERS, extrasaction='ignore')
    writer.writeheader()
    for dos in senat_csv:
        writer.writerow(dos)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys, os, time, re, requests, io
from datetime import date, datetime
from html.entities import name2codepoint
from csv import DictReader
from difflib import SequenceMatcher
import json
import locale
import contextlib

from diff_match_patch import diff_match_patch


locale.setlocale(locale.LC_TIME, 'fr_FR.utf-8')

from .sort_articles import bister

from lawfactory_utils.urls import download


@contextlib.contextmanager
def log_print(file=None, only_log=False):
    """capture all outputs to a log file while still printing it"""
    class Logger:
        def __init__(self, file):
            self.terminal = sys.stdout
            self.log = file
            self.only_log = only_log

        def write(self, message):
            if not self.only_log:
                self.terminal.write(message)
            self.log.write(message)

        def __getattr__(self, attr):
            return getattr(self.terminal, attr)

    if file is None:
        file = io.StringIO()

    logger = Logger(file)

    _stdout = sys.stdout
    _stderr = sys.stderr
    sys.stdout = logger
    sys.stderr = logger
    try:
        yield logger.log
    finally:
        sys.stdout = _stdout
        sys.stderr = _stderr


def open_csv(dirpath, filename, delimiter=";"):
    try:
        data = []
        with open(os.path.join(dirpath, filename), 'r') as f:
            for row in DictReader(f, delimiter=delimiter):
                data.append(dict([(k, v) for k, v in row.items()]))
            return data
    except Exception as e:
        print(type(e), e, file=sys.stderr)
        sys.stderr.write("ERROR: Could not open file %s in dir %s\n" % (filename, dirpath))
        raise e


def open_json(dirpath, filename=None):
    if filename:
        path = os.path.join(dirpath, filename)
    else:
        path = dirpath
    try:
        with open(path, 'r') as f:
            return json.load(f)
    except Exception as e:
        print(type(e), e, file=sys.stderr)
        sys.stderr.write("ERROR: Could not open file %s" % (path,))
        raise e


def download_daily(url_or_collecter, filename, output_directory):
    if not os.path.exists(output_directory):
        os.makedirs(output_directory)
    yesterday = time.time() - 86400
    destfile = os.path.join(output_directory, filename + ".json")
    if not os.path.exists(destfile) or os.path.getmtime(destfile) < yesterday:
        print('downloading', filename)
        if isinstance(url_or_collecter, str):
            jsondata = download(url_or_collecter).json()
        else:
            jsondata = url_or_collecter()
        print_json(jsondata, destfile)
    else:
        jsondata = open_json(destfile)
    return jsondata


def print_json(dico, filename=None):
    jdump = json.dumps(dico, ensure_ascii=False, sort_keys=True, indent=2)
    if filename:
        # create parent directories if necessary
        dirname = os.path.dirname(filename)
        if dirname:
            os.makedirs(dirname, exist_ok=True)

        try:
            with open("%s.tmp" % filename, 'w') as f:
                f.write(jdump)
            if os.path.exists(filename):
                os.remove(filename)
            os.rename("%s.tmp" % filename, filename)
        except Exception as e:
            print(type(e), e, file=sys.stderr)
            sys.stderr.write("ERROR: Could not write in file %s" % filename)
            raise e
    else:
        print(jdump)


def debug_file(data, filename):
    if '--debug' in sys.argv:
        path = os.path.join('debug', filename)
        print_json(data, path)
        print('   DEBUG - dumped', path)


datize = lambda d: date(*tuple([int(a) for a in d.split('-')]))
def format_date(d):
    da = d.split('/')
    da.reverse()
    return "-".join(da)

def format_display_date(d):
    return datetime.strftime(datize(d), '%A %d %B %Y').replace(' 0', ' ')

upper_first = lambda t: t[0].upper() + t[1:] if len(t) > 1 else t.upper()

re_entities = re.compile(r'&([^;]+)(;|$)')
decode_char = lambda x: chr(int(x.group(1)[1:]) if x.group(1).startswith('#') else name2codepoint[x.group(1)])
decode_html = lambda text: re_entities.sub(decode_char, text)

re_clean_spaces = re.compile(r"[\s\n]+")
clean_spaces = lambda x: re_clean_spaces.sub(" ", x)

re_clean_balises = re.compile(r"<\/?[!a-z][^>]*>", re.I)
clean_balises = lambda x: re_clean_balises.sub("", x)

clean_statuses = lambda x: re_alin_sup.sub("", x)

strip_text = lambda x: clean_spaces(clean_statuses(clean_balises(x))).strip()

re_non_alphanum = re.compile(r"[^a-z0-9]+")

upcase_accents = "ÇÀÂÄÉÈÊËÎÏÔÖÙÛÜ"
locase_accents = "çàâäéèêëîïôöùûü"
case_noaccents = "caaaeeeeiioouuu"

def real_lower(text):
    for a in upcase_accents:
        text = text.replace(a, locase_accents[upcase_accents.find(a)])
    return text.lower()

def clean_accents(text):
    text = real_lower(text)
    for a in locase_accents:
        text = text.replace(a, case_noaccents[locase_accents.find(a)])
    return text

re_clean_alin = re.compile(r'^"?(([IVXCDLM]+|\d+|[a-z])[°)\-\.\s]+)+\s*((%s|[A-Z%s]+)[°)\-\.\s]+)*' % (bister, upcase_accents))
re_alin_sup = re.compile(r'\s*\((censur|supprim)és?\)$', re.I)

def clean_text_for_diff(text):
    if type(text) == list:
        text = [clean_statuses(re_clean_alin.sub('', t)) for t in text]
        text = "\n".join([t for t in text if t])
    else:
        text = clean_statuses(re_clean_alin.sub('', text))
    text = clean_balises(text)
    text = clean_accents(text)
    text = re_non_alphanum.sub('', text)
    return text

def compute_approx_similarity(text1, text2):
    a = SequenceMatcher(None, text1, text2, autojunk=False)
    return 1 - a.real_quick_ratio()

def compute_similarity(text1, text2):
    dmp = diff_match_patch()
    dmp.Diff_Timeout = 0
    diff = dmp.diff_main(text1, text2)

    # similarity
    common_text = sum([len(txt) for op, txt in diff if op == 0])
    text_length = max(len(text1), len(text2))
    sim = common_text / text_length
    return sim

def compute_similarity_by_articles(text1, text2):
    dmp = diff_match_patch()
    dmp.Diff_Timeout = 0
    arts = set(text1.keys()) | set (text2.keys())
    common_text = 0
    text_length = 0
    for a in arts:
        if a in text1 and a in text2:
            diff = dmp.diff_main(text1[a], text2[a])
            common_text += sum([len(txt) for op, txt in diff if op == 0])
            text_length += max(len(text1[a]), len(text2[a]))
        elif a in text1:
            text_length += len(text1[a])
        else:
            text_length += len(text2[a])
    sim = common_text / text_length
    return sim


def identify_room(url_or_institution, legislature):
    typeparl = "depute" if 'nationale.fr' in url_or_institution \
        or url_or_institution == 'assemblee' else "senateur"
    if typeparl == 'depute':
        year = 1942 + 5*legislature
        legis = '%s-%s' % (year, year+5)
    else:
        legis = 'www'
    urlapi = "%s.nos%ss" % (legis, typeparl)
    return typeparl, urlapi


def national_assembly_text_legislature(url_text):
    return int(url_text.split('.fr/')[1].split('/')[0])


def personalize_link(link, obj, urlapi):
    if isinstance(obj, dict):
        slug = obj.get('intervenant_slug', obj.get('slug', ''))
    else: slug = obj
    typeparl = "senateur" if urlapi.endswith("senateurs") else "depute"
    if slug:
        return link.replace("##URLAPI##", urlapi).replace("##TYPE##", typeparl).replace("##SLUG##", slug)
    return ""

parl_link = lambda obj, urlapi: personalize_link("https://##URLAPI##.fr/##SLUG##", obj, urlapi)
photo_link = lambda obj, urlapi: personalize_link("https://##URLAPI##.fr/##TYPE##/photo/##SLUG##", obj, urlapi)
groupe_link = lambda obj, urlapi: personalize_link("https://##URLAPI##.fr/groupe/##SLUG##", obj, urlapi)
amdapi_link = lambda urlapi: personalize_link("https://##URLAPI##.fr/api/document/Amendement/", {'slug': 'na'}, urlapi)

def slug_groupe(g):
    # on the senat side, some senators have obsolete groups
    g = g.upper()
    g = g.replace("SOCV", "SOC")
    g = g.replace("CRC-SPG", "CRC")
    g = g.replace("ECOLO", "ECO")
    g = g.replace("ECO", "ECOLO")
    return g


class Context(object):

    def __init__(self, sourcedir, load_parls=False):
        self.sourcedir = sourcedir
        if not self.sourcedir:
            sys.stderr.write('ERROR: no input directory given\n')
            exit(1)
        self.allgroupes = {}
        self.get_groupes()
        self.get_historique_groupes_senat()
        self.parlementaires = {}
        if load_parls:
            self.get_parlementaires()

    def get_procedure(self):
        try:
            with open(os.path.join(self.sourcedir, 'viz', 'procedure.json'), "r") as procedure:
                return json.load(procedure)
        except Exception as e:
            sys.stderr.write('ERROR: could not find procedure data in directory %s\n' % self.sourcedir)
            raise e

    def get_parlementaires(self):
        for f in os.listdir(os.path.join(self.sourcedir, '..')):
            if f.endswith('.parlementaires.json'):
                url = f.replace('.parlementaires.json', '').lower()
                try:
                    with open(os.path.join(self.sourcedir, '..', f), "r") as parls:
                        self.parlementaires[url] = {}
                        typeparl = "depute" if "depute" in url else "senateur"
                        for parl in json.load(parls)[typeparl+"s"]:
                            p = parl[typeparl]
                            self.parlementaires[url][p["slug"]] = p
                except Exception as e:
                    sys.stderr.write('WARNING: could not read parlementaires file %s in data\n' % f)
                    sys.stderr.write('%s: %s\n' % (type(e), e))

    def get_parlementaire(self, urlapi, slug):
        try:
            return self.parlementaires[urlapi][slug]
        except:
            typeparl = "depute" if "deputes" in urlapi else "senateur"
            if urlapi not in self.parlementaires:
                self.parlementaires[urlapi] = {}
            self.parlementaires[urlapi][slug] = requests.get(parl_link(slug, urlapi)+"/json").json()[typeparl]
            return self.parlementaires[urlapi][slug]

    def get_groupes(self):
        for f in os.listdir(os.path.join(self.sourcedir, '..')):
            if f.endswith('-groupes.json'):
                url = f.replace('-groupes.json', '').lower()
                try:
                    with open(os.path.join(self.sourcedir, '..', f), "r") as gpes:
                        self.allgroupes[url] = {}
                        for gpe in json.load(gpes)['organismes']:
                            if not gpe["organisme"]["acronyme"]:
                                continue
                            acro = slug_groupe(gpe["organisme"]["acronyme"])
                            self.allgroupes[url][acro] = {
                                "nom": gpe["organisme"]['nom'],
                                "order": int(gpe["organisme"]['order']),
                                "color": "rgb(%s)" % gpe["organisme"]['couleur']}
                except Exception as e:
                    sys.stderr.write('WARNING: could not read groupes file %s in data\n' % f)
                    sys.stderr.write('%s: %s\n' % (type(e), e))

    def get_historique_groupes_senat(self):
        senateurs = {}
        gfile = os.path.join(self.sourcedir, '..', 'historique-groupes-senat.json')
        for row in open_json(gfile)["results"]:
            sid = row['Matricule'].lower()
            if sid not in senateurs:
                senateurs[sid] = []
            st = row.get('Date_de_debut_de_la_fonction') or row.get('Date_de_debut_d_appartenance') or '0000-00-00'
            ed = row.get('Date_de_fin_de_la_fonction') or row.get('Date_de_fin_d_appartenance') or '9999-99-99'
            senateurs[sid].append({
                'debut': st[:10].replace('/', '-'),
                'fin': ed[:10].replace('/', '-'),
                'groupe': slug_groupe(row['Code_du_groupe_politique'])
            })
        for sid in senateurs:
            senateurs[sid] = sorted(senateurs[sid], key=lambda x: (x['debut'], x['fin']))
        self.groupes_senateurs = senateurs

    def get_senateur_groupe(self, slug, object_date, urlapi):
        senator = self.get_parlementaire(urlapi, slug)
        senid = senator["id_institution"][-6:].lower()
        if senid not in self.groupes_senateurs:
            print('WARNING - cannot find %s in OpenData Sénat' % slug)
        else:
            for period in self.groupes_senateurs[senid]:
                if period["debut"] <= object_date <= period["fin"]:
                    return period["groupe"]
            print('WARNING - cannot find groupe of %s in OpenData Sénat for date %s' % (slug, object_date))
            for period in self.groupes_senateurs[senid]:
                print(' > ', period['debut'], period['fin'], period['groupe'])
        return None

    def add_groupe(self, groupes, gpe, urlapi):
        gpid = upper_first(gpe.lower())
        acro = slug_groupe(gpid)
        if acro in self.allgroupes[urlapi]:
            gpid = acro
        if gpid not in groupes:
            groupes[gpid] = {'nom': upper_first(gpe),
                             'link': ''}
            if gpid in self.allgroupes[urlapi]:
                groupes[gpid]['nom'] = self.allgroupes[urlapi][gpid]['nom']
                groupes[gpid]['order'] = 10 + self.allgroupes[urlapi][gpid]['order']
                groupes[gpid]['color'] = self.allgroupes[urlapi][gpid]['color']
                groupes[gpid]['link'] = groupe_link({'slug': gpid}, urlapi)
            elif gpid == "Présidence":
                groupes[gpid]['color'] = "#bfbbcc"
                groupes[gpid]['order'] = 0
            elif gpid == "Rapporteurs":
                groupes[gpid]['color'] = "#b9ccc0"
                groupes[gpid]['order'] = 50
            elif gpid == "Gouvernement":
                groupes[gpid]['color'] = "#cccbb3"
                groupes[gpid]['order'] = 60
            elif gpid == "Auditionnés":
                groupes[gpid]['color'] = "#ccb7b6"
                groupes[gpid]['order'] = 70
            else:
                groupes[gpid]['color'] = "#bfbfbf"
                groupes[gpid]['order'] = 100
        return gpid


def get_text_id(texte_url):
    if "nationale.fr" in texte_url:
        textid_match = re.search(r'fr\/(\d+)\/.*[^0-9]0*([1-9][0-9]*)(-a\d)?\.asp$', texte_url, re.I)
        nosdeputes_id = textid_match.group(2)
        if '/ta/ta' in texte_url:
            nosdeputes_id = 'TA' + nosdeputes_id
        return nosdeputes_id
    elif "senat.fr" in texte_url:
        textid_match = re.search(r"(\d{2})-(\d+)(/|rec|(_mono)?\.html$)", texte_url, re.I)
        return '20%s20%s-%s' % (textid_match.group(1).zfill(2),
            str(int(textid_match.group(1))+1).zfill(2),
            textid_match.group(2))



#!/usr/bin/env python
# -*- coding: utf-8 -*-


def detect_auteur_depot_from_url(url):
    if '/propositions/' in url:
        return "Députés"
    elif '/leg/ppl' in url:
        return "Sénateurs"
    return "Gouvernement"


def add_auteur_depot(step):
    if step.get('step', '') == 'depot' and step['debats_order'] is not None:
        step['auteur_depot'] = detect_auteur_depot_from_url(step.get('source_url', ''))


def remove_interventions_too_small(step, intervs):
    # TODO: this warning is never triggered, maybe this code is obsolete ?
    if step.get('has_interventions') and step['directory'] not in intervs:
        print("[warning] [update_procedure] removing nearly empty interventions steps for", step['directory'])
        step['has_interventions'] = False


def process(procedure, articles, intervs={}):
    """
    Process procedure to
        - add a fake `enddate` where it's useful
        - detect the steps we want to display
            (add a `debats_order` to those steps)
        - add `auteur_depot`
        - remove extra predicted steps
    """
    articles = articles['articles']

    good_steps = {}
    for _, a in articles.items():
        for s in a['steps']:
            stepid = s['directory']
            if stepid not in good_steps:
                good_steps[stepid] = int(s['directory'].split('_')[0])

    if not good_steps:
        raise Exception('[update_procedure] no steps to display, parsing must have failed')

    currently_debated_step = None
    for s in procedure['steps']:
        if s.get('in_discussion'):
            currently_debated_step = s

        if 'enddate' not in s and currently_debated_step is None:
            s['enddate'] = s.get('date')

        s['debats_order'] = None
        if 'directory' in s:
            if currently_debated_step == s and good_steps:
                s['debats_order'] = max(good_steps.values()) + 1
            elif not currently_debated_step:
                s['debats_order'] = good_steps.get(s['directory'], None)

        remove_interventions_too_small(s, intervs)
        add_auteur_depot(s)

    # remove predicted steps after the one in discussion
    if not procedure.get('url_jo'):
        steps_to_keep = []
        for s in procedure['steps']:
            if s.get('stage') == 'promulgation':
                break
            steps_to_keep.append(s)
            if s.get('in_discussion'):
                break
        else:
            raise Exception("[update_procedure] no step in discussion for live text")
        procedure['steps'] = steps_to_keep

    return procedure


if __name__ == "__main__":
    # test: do not mark promulgation step as visible step even for
    #       unfinished texts (#95)
    result = process(
        {
            "url_jo": None,
            "steps": [
                {
                    "date": "2011-12-01",
                    "directory": "00_",
                    "enddate": "2011-12-01",
                    "institution": "senat",
                    "stage": "1ère lecture",
                    "step": "depot",
                },
                {
                    "date": "2011-12-13",
                    "directory": "02_",
                    "institution": "senat",
                    "stage": "1ère lecture",
                    "step": "hemicycle",
                },
                {
                    "date": None,
                    "directory": "03_",
                    "institution": "gouvernement",
                    "stage": "promulgation",
                },
            ],
        },
        {
            "articles": {
                "1er": {
                    "steps": [
                        {"directory": "00_"},
                        {"directory": "01_"},
                        {"directory": "02_"},
                    ]
                }
            }
        },
    )

    try:
        assert result == {
            "steps": [
                {
                    "auteur_depot": "Gouvernement",
                    "date": "2011-12-01",
                    "debats_order": 0,
                    "directory": "00_",
                    "enddate": "2011-12-01",
                    "institution": "senat",
                    "stage": "1ère lecture",
                    "step": "depot",
                },
                {
                    "date": "2011-12-13",
                    "debats_order": 2,
                    "directory": "02_",
                    "enddate": "2011-12-13",
                    "institution": "senat",
                    "stage": "1ère lecture",
                    "step": "hemicycle",
                }
            ],
            "url_jo": None,
        }
    except Exception as e:
        import pprint
        pprint.pprint(result)
        raise e
    print("[update_procedure] TESTS OK")



import sys, os

from .common import open_json


procedure_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'valid_procedure.json')
procedure = open_json(procedure_file)

def find_anomalies(dossiers, verbose=True):
    anomalies = 0
    for dos in dossiers:
        prev_step = ''
        for step in dos['steps']:
            step_name = ' • '.join((x for x in (step.get('stage'), step.get('institution'), step.get('step','')) if x))
            if procedure.get(prev_step, {}).get(step_name, False) is False:
                if verbose:
                    print('INCORRECT', prev_step, '->', step_name)
                    print(dos.get('url_dossier_senat'), '|',dos.get('url_dossier_assemblee'))
                    print()
                anomalies += 1

            #print(step_name, '      \t\t\t\t===>>', procedure.get(prev_step, {}).get(step_name))

            prev_step = step_name

    if verbose and anomalies:
        print(anomalies, 'anomalies (', len(dossiers), 'doslegs)')
    return anomalies

"""
TODO:
 - anomalies liens: /leg/tas for textes sénat
"""

if __name__ == '__main__':
    if not os.isatty(0):
        import json
        dossiers = json.loads(sys.stdin.read())
        if type(dossiers) is not list:
            dossiers = [dossiers]
    else:
        dossiers = open_json(sys.argv[1])
    find_anomalies(dossiers)



import copy

from tlfp.tools.detect_anomalies import find_anomalies
from tlfp.tools._step_logic import should_ignore_commission_text, use_old_procedure

from lawfactory_utils.urls import validate_link_CC_decision


def fix_an_cmp_step_url(senat, an):
    # detect missing AN CMP step in senat data
    dos = copy.deepcopy(senat)

    # if CMP.assemblee.hemicycle is empty in senat data but ok in AN data
    # complete from AN data
    an_cmp_step_from_senat = [i for i, step in enumerate(senat['steps']) if
        step.get('stage') == 'CMP' and step.get('institution') == 'assemblee']
    an_cmp_step_from_an = [i for i, step in enumerate(an['steps']) if
        step.get('stage') == 'CMP' and step.get('institution') == 'assemblee']
    if an_cmp_step_from_an:
        if an_cmp_step_from_senat:
            dos['steps'][an_cmp_step_from_senat[0]]['source_url'] = \
                an['steps'][an_cmp_step_from_an[0]].get('source_url')

    return dos


def merge_promulgation_steps(senat_step, an_step):
    step = {**senat_step}
    senat_url = senat_step.get('source_url')
    if not senat_url or 'jo_pdf' in senat_url and an_step.get('source_url'):
        step['source_url'] = an_step['source_url']
    if not senat_step.get('date') and an_step.get('date'):
        step['date'] = an_step['date']
    return step


def merge_senat_with_an(senat, an):
    """Takes a senat dosleg and an AN dosleg and returns a merged version"""
    dos = copy.deepcopy(senat)

    dos['url_dossier_assemblee'] = an['url_dossier_assemblee']
    dos['assemblee_slug'] = an['assemblee_slug']
    dos['assemblee_id'] = an['assemblee_id']
    dos['assemblee_legislature'] = an['assemblee_legislature']

    if ('url_jo' not in dos or 'jo_pdf' in dos['url_jo']) and 'url_jo' in an:
        dos['url_jo'] = an['url_jo']
        if not dos.get('end') and an.get('end'):
            dos['end'] = an['end']

    dos['steps'] = []
    an['steps'] = [s for s in an['steps'] if not should_ignore_commission_text(s, an)]

    def same_stage_step_instit(a, b):
        return a.get('stage') == b.get('stage') and a.get('step') == b.get('step') \
            and a.get('institution') == b.get('institution')

    empty_last_step = False
    an_offset = 0
    for i, step in enumerate(senat['steps']):
        steps_to_add = []

        # generate an offset when steps are missing on the AN side
        if step.get('institution') == 'senat':
            an_index = i + an_offset
            if len(an['steps']) > an_index and an_index > 0:
                an_step = an['steps'][an_index]
                if not same_stage_step_instit(an_step, step):
                    found_same_step_at = None
                    for j, next_senat in enumerate(senat['steps'][i:]):
                        if same_stage_step_instit(next_senat, an_step) \
                            and next_senat.get('source_url') == an_step.get('source_url'):
                            found_same_step_at = j
                            break
                    if found_same_step_at:
                        an_offset -= found_same_step_at

        # complete senat data from AN
        if step.get('institution') == 'assemblee':
            an_index = i + an_offset
            if len(an['steps']) > an_index and an_index >= 0:
                an_step = an['steps'][an_index]

                # get data from AN even if there's data on the senat side
                if same_stage_step_instit(an_step, step):
                    # only take source_url from the AN, we've got better infos for now
                    # from the senat
                    common_step = copy.deepcopy(step)
                    if an_step.get('source_url') and (
                      # In lecture définitive's dépot, if one source_url is an AN TA, it is probably the good one and should be kept if the other one is a PL
                      not (step.get('stage') == 'l. définitive' and step.get('step') == 'depot' and '/projets/pl' in an_step.get('source_url') and '/ta/ta' in step['source_url'])):
                        common_step['source_url'] = an_step.get('source_url')
                    if not common_step.get('date'):
                        common_step['date'] = an_step.get('date')
                    steps_to_add.append(common_step)

                    """
                    try to find "holes":
                        - we are in an AN step
                        - next step is different
                        - a few steps later, it's the same !
                    """
                    if len(an['steps']) > an_index+1 and len(senat['steps']) > i+1:
                        next_step_an = an['steps'][an_index+1]
                        next_step = senat['steps'][i+1]
                        if not same_stage_step_instit(next_step_an, next_step):
                            found_same_step_at = None
                            for j, next_an in enumerate(an['steps'][an_index+1:]):
                                if next_an.get('institution') != 'senat' and same_stage_step_instit(next_an, next_step):
                                    found_same_step_at = j
                                    break

                            if found_same_step_at is not None:
                                steps_to_add = an['steps'][an_index:an_index+j+1]
                                an_offset += found_same_step_at

        # CMP commission: get extra urls from AN side if there's a different one
        if step.get('stage') == 'CMP' and step.get('step') == 'commission':
            for an_index, an_step in enumerate(an['steps']):
                if same_stage_step_instit(an_step, step) and an_step.get('source_url'):
                    an_offset = an_index - i
                    if 'cmp_commission_other_url' in an_step:
                        if an_step['cmp_commission_other_url'] == step['source_url']:
                            step['cmp_commission_other_url'] = an_step['source_url']
                        else:
                            step['cmp_commission_other_url'] = an_step['cmp_commission_other_url']
                    elif step.get('source_url'):
                        if step['source_url'] != an_step['source_url']:
                            step['cmp_commission_other_url'] = an_step['source_url']
                    else:
                        step['source_url'] = an_step['source_url']
            if dos.get('url_jo') and not step.get('source_url'):
                print('[warning] [merge] empty CMP steps announced in Senate in a promulgated text but missing in the AN-side', dos['url_dossier_assemblee'])
                continue

        # Choose best CC url available
        if step.get('stage') == 'constitutionnalité' and not validate_link_CC_decision(step.get('source_url')):
            cc_an = [s.get('source_url') for s in an['steps'] if s.get('stage') == 'constitutionnalité']
            if validate_link_CC_decision(cc_an[0]):
                step['source_url'] = cc_an[0]

        # Choose best JO url available
        if step.get('stage') == 'promulgation':
            an_step_promulgation = [(i, s) for i, s in enumerate(an['steps']) if s.get('stage') == 'promulgation']
            if an_step_promulgation:
                an_index, an_step = an_step_promulgation[0]
                an_offset = an_index - i
                step = merge_promulgation_steps(step, an_step)

        # Only keep first empty consecutive step as next one to come
        if not dos.get('url_jo'):
            if step.get('source_url'):
                empty_last_step = False
            elif empty_last_step:
                break
            else:
                if not step.get('step') == 'commission' and use_old_procedure(step, dos):
                    empty_last_step = True

        if len(steps_to_add) == 0:
            steps_to_add = [step]

        dos['steps'] += steps_to_add

    dos = fix_an_cmp_step_url(dos, an)

    # ## SANITY CHECKS ## #

    # detect AN leftovers we didn't merge
    if len(an['steps']) > i + an_offset + 1:
        leftovers = an['steps'][an_index:]
        if not dos.get('url_jo'):
            dos['steps'] += leftovers
        else:
            print('[warning] [merge] some AN steps didn\'t get merged ( the last', len(leftovers), ')')

    # compare the number of anomalies before and after merging
    if find_anomalies([senat], verbose=False) < find_anomalies([dos], verbose=False) or \
       find_anomalies([an], verbose=False) < find_anomalies([dos], verbose=False):
        print('[warning] [merge] more anomalies in the steps after the merge:', dos['url_dossier_senat'])

    # verify number of the CMP steps didn't move while merging
    cmp_steps_in_merged_dos = len([1 for step in dos['steps'] if step.get('stage') == 'CMP'])
    cmp_steps_in_senat_dos = len([1 for step in senat['steps'] if step.get('stage') == 'CMP'])
    if cmp_steps_in_merged_dos != cmp_steps_in_senat_dos:
        # since we remove the Senate predicted CMP steps, CMP steps can disappear for live texts
        if dos.get('url_jo') or cmp_steps_in_merged_dos > cmp_steps_in_senat_dos:
            print('[warning] [merge] number of CMP steps changed', dos['url_dossier_senat'],
                  '(before:', cmp_steps_in_senat_dos, ', after:', cmp_steps_in_merged_dos, ')')

    return dos



import re

from lawfactory_utils.urls import download
from senapy.dosleg.parser import parse as senapy_parse

from .tools import parse_texte, complete_articles
from .tools._step_logic import get_previous_step, use_old_procedure, is_one_of_the_initial_depots, should_ignore_commission_text
from .tools.common import debug_file


def test_status(url):
    resp = download(url)
    if resp.status_code != 200:
        return False
    # TODO: do this in download()
    if 'assemblee-nationale.fr' in url:
        resp.encoding = 'Windows-1252'
    return resp


def is_step_in_discussion(steps, step_index):
    for s in steps[step_index + 1:]:
        if 'source_url' in s and s.get('step') in (
            'hemicycle',
            'commission',
            'constitutionnalité'
        ):
            return False
    return True


def find_good_url_resp(url):
    if 'senat.fr' in url:
        # Depot steps can sometime link a previous abandonned dosleg
        # ex: http://www.senat.fr/dossier-legislatif/ppl09-338.html
        if '/dossier-legislatif/' in url:
            resp = test_status(url)
            if resp:
                dos = senapy_parse(resp.text, url)
                return find_good_url_resp(dos['steps'][0]['source_url'])

        if '/leg/' in url and url.endswith('.html'):
            resp = test_status(url)
            if resp:
                return resp
        if '/rap/' in url:
            # we try to use the last page to get a clean text
            clean_url_resp = None
            for page in '0123456789':
                new_url = url.replace('.html', page + '.html')
                resp = test_status(new_url)
                if not resp:
                    break
                text = resp.text.replace('<br>', '\n')
                # look for the "TEXTE ÉLABORÉ PAR .."" TITLE
                if re.match(r'.*TEXTE\s+&Eacute;LABOR&Eacute;\s+PAR.*', text, re.M | re.DOTALL) \
                    or re.search(r'>PROJET DE LOI ', text) \
                    or re.match(r'.*EXAMEN\s+EN\s+COMMISSION.*', text, re.M | re.DOTALL):
                    # if the previous page was valid also, then the text is multi-page
                    if clean_url_resp:
                        clean_url_resp = None
                        break
                    clean_url_resp = resp

            if clean_url_resp:
                return clean_url_resp
            else:
                # use _mono as last resort
                mono_url = url.replace('.html', '_mono.html')
                resp = test_status(mono_url)
                if resp:
                    return resp

    if 'assemblee-nationale.fr' in url:
        if '/cr-' in url:
            return False
        if '/dossiers/' in url:
            return False

        if 'documents/notice/' in url:
            url = url.replace('www2', 'www').replace('documents/notice/', '').split('/(index)')[0] + '.asp'

        if url.endswith('.pdf'):
            url = url.replace('/pdf/', '/propositions/').replace('.pdf', '.asp')

        """
        # /textes/ URL are not supported yet in parse_text
        if '/ta-commission/' in url:
            text_id = url.split('/')[-1].split('-')[0].replace('r', '')
            new_url = url.split('/ta-commission/')[0] + '/textes/' + text_id + '.asp'
            if test_status(new_url):
                return new_url

        if '/projets/' in url:
            new_url = url.replace('/projets/pl', '/textes/')
            if test_status(new_url):
                return new_url

        if '/propositions/' in url:
            new_url = url.replace('/propositions/pion', '/textes/')
            if test_status(new_url):
                return new_url

        if '/ta/' in url:
            new_url = url.replace('/ta/ta', '/textes/')
            if test_status(new_url):
                return new_url
        """

        resp = test_status(url)
        if not resp \
            or "n'est pas encore édité" in resp.text \
            or ">Cette division n'est pas encore distribuée<" in resp.text:
            return False
        else:
            return resp

    resp = test_status(url)
    if resp:
        return resp
    return False


def parse_url_for_step(url, step, step_index, retry=True):
    fixed_url_resp = find_good_url_resp(url)
    if fixed_url_resp:
        fixed_url = fixed_url_resp.url
        if fixed_url != url:
            print('        ^ text url fixed:', fixed_url)

        if step.get('stage') != 'constitutionnalité':
            step['source_url'] = fixed_url

        articles = parse_texte.parse(fixed_url, resp=fixed_url_resp)
        debug_file(articles, 'parsed_text_step_%d.json' % step_index)

        if (not articles or len(articles) < 2):
            if retry:
                if 'ta-commission' in fixed_url:
                    fixed_url = fixed_url.replace('ta-commission', 'rapports').replace('-a0', '')
                    print('        ^ empty parsing, trying url rapport :', fixed_url)
                    return parse_url_for_step(fixed_url, step, step_index, retry=False)
                if 'assemblee-nationale' in fixed_url and 'rapports' in fixed_url:
                    fixed_url = fixed_url.replace('rapports', 'ta-commission').replace('.asp', '-a0.asp')
                    print('        ^ empty parsing, trying url TA :', fixed_url)
                    return parse_url_for_step(fixed_url, step, step_index, retry=False)

            raise Exception('[parse_texts] Empty parsing %s' % url)

        step['articles'] = articles
        text = articles[0]
        text['depot'] = step.get('step') == 'depot'

        # echec detected ? we update the step
        echec_line = [article for article in articles if article.get('type') == 'echec']
        if echec_line:
            assert step.get('step') != 'depot'
            echec_line = echec_line[0]
            if step.get('stage') == 'CMP':
                step['echec'] = 'échec'
            else:
                step['echec'] = 'rejet'

        if not step.get('echec') and len(articles) == 1:
            re_rapport_senat = re.compile(r'(senat.fr/rap/([^/]+)/\2)\d.html')
            if re_rapport_senat.search(fixed_url):
                fixed_url = re_rapport_senat.sub(r'\1_mono.html', fixed_url)
                print('        ^ empty parsing, trying url mono :', fixed_url)
                return parse_url_for_step(fixed_url, step, step_index)

            raise Exception('parsing failed for %s (no text)' % fixed_url)
    elif 'ta-commission' in url:
        fixed_url = url.replace('ta-commission', 'rapports').replace('-a0', '')
        print('        ^ empty response, trying url rapport :', fixed_url)
        parse_url_for_step(fixed_url, step, step_index)
    else:
        raise Exception('[parse_texts] Invalid response %s' % url)


def parse_texts(dos):
    print('** parsing texts')

    steps = dos['steps']

    # first parse the texts
    for step_index, step in enumerate(steps):
        url = step.get('source_url')
        print('    ^ text: ', url)

        if should_ignore_commission_text(step, dos):
            continue

        # we parse the JO texte only if there's a CC decision
        if step.get('stage') == 'constitutionnalité':
            url = dos.get('url_jo')
        if step.get('stage') == 'promulgation':
            continue


        try:
            if url is None:
                if step.get('echec') is None:
                    raise Exception('[parse_texts] Empty url for step: %s.%s.%s' % (step.get('institution'), step.get('stage'), step.get('step')))
                continue
            else:
                parse_url_for_step(url, step, step_index)
        except Exception as e:
            if step.get('step') == 'depot' and not is_one_of_the_initial_depots(steps, step_index):
                print('     * ignore missing intermediary depot', url)
                continue

            if not dos.get('url_jo') and is_step_in_discussion(steps, step_index):
                step['in_discussion'] = True
                print('     * ignore step in discussion')
                break

            raise e


def re_order_cmp(dos):
    # re-order CMPs via texte définitif detection
    steps = dos['steps']

    cmp_hemi_steps = [i for i, step in enumerate(dos['steps']) if
        step.get('stage') == 'CMP' and step.get('step') == 'hemicycle']

    if cmp_hemi_steps and len(cmp_hemi_steps) == 2:
        first_i, second_i = cmp_hemi_steps
        first = dos['steps'][first_i]
        if first.get('articles', [{}])[0].get('definitif') or first.get('echec'):
            print('     * re-ordered CMP steps')
            steps = dos['steps']
            steps[first_i], steps[second_i] = steps[second_i], steps[first_i]


def complete_texts(dos):
    steps = dos['steps']
    for step_index, step in enumerate(steps):
        print('    ^ complete text: ', step.get('source_url'))

        if 'articles' in step:
            old_proc = use_old_procedure(step, dos)
            prev_step_index = get_previous_step(steps, step_index, old_proc)
            if prev_step_index is not None and not step.get('echec'):
                if is_one_of_the_initial_depots(steps, step_index):
                    step['articles_completed'] = step['articles']
                else:
                    # get ante-previous step for hemicycle text where an alinea
                    # can reference the depot step instead of the commission text
                    anteprevious = None
                    if step.get('step') == 'hemicycle' and steps[prev_step_index].get('step') == 'commission':
                        antestep_index = get_previous_step(steps, prev_step_index, old_proc, get_depot_step=True)
                        if antestep_index is not None and steps[antestep_index].get('step') == 'depot':
                            anteprevious = steps[antestep_index].get(
                                'articles_completed',
                                steps[antestep_index].get('articles', [])
                            )

                    prev_step = steps[prev_step_index]
                    real_prev_step = steps[step_index-1]
                    real_prev_step_arts = real_prev_step.get(
                        'articles_completed',
                        real_prev_step.get('articles',
                            [{"type": "texte",
                             "id": real_prev_step.get("source_url", "").replace('.asp', '').replace('.html', ''),
                             "skip": old_proc or real_prev_step.get("echec")}]
                        )
                    )
                    real_prev_step_metas = real_prev_step_arts[0] if real_prev_step_arts else {}
                    complete_args = {
                        'current': step.get('articles', []),
                        'previous': prev_step.get(
                            'articles_completed',
                            prev_step.get('articles', [])
                        ),
                        'step': step,
                        'previous_step_metas': real_prev_step_metas,
                        'table_concordance': dos.get('table_concordance', {}),
                        'anteprevious': anteprevious,
                    }
                    debug_file(complete_args, 'complete_args_step_%d.json' % step_index)
                    step['articles_completed'] = complete_articles.complete(**complete_args)
                    debug_file(step.get('articles_completed'), 'completed_text_step_%d.json' % step_index)


def process(dos):
    # TODO(cleanup): articles_completed/articles attributes are hacky
    parse_texts(dos)
    re_order_cmp(dos)
    complete_texts(dos)
    return dos



import sys

from legipy.services.law_service import LawService

from tlfp.tools.common import download_daily


def process(output_directory):
    return download_daily(
        lambda : {l.id_legi: l.common_name for l in LawService().common_laws()},
        'lois_dites',
        output_directory
    )


if __name__ == '__main__':
    process(sys.argv[1])



import os, sys

from tlfp.tools.common import *


def list_get_or_none(arr, i):
    if 0 <= i < len(arr):
        return arr[i]

def process(OUTPUT_DIR, procedure):
    context = Context(OUTPUT_DIR)
    steps = procedure['steps']
    for step_i, step in enumerate(steps):

        if 'intervention_files' not in step:
            continue

        interventions_files_moved = []
        for interv_file in step['intervention_files']:
            interv_dirpath = os.path.join(context.sourcedir, 'procedure', step['directory'], 'interventions')
            interv = open_json(interv_dirpath, "%s.json" % interv_file)['seance'][0]['intervention']
            date = interv['date']

            prev_step = list_get_or_none(steps, step_i-1)
            next_step = list_get_or_none(steps, step_i+1)

            # if step date is later than intervention date, use intervention date
            if step.get('date') and step['date'] > date:
                if not prev_step or prev_step['date'] <= date:
                    step['date'] = date
                    print('INFO: change beginning date of', step_i, 'thanks to', interv_file)
                else:
                    print('ERROR: PB date of', interv_file, ', step begins', step['date'], 'and prev date ends', prev_step['enddate'])

            # if enddate is earlier than end of interventions, use interventions date
            if step.get('enddate') and step['enddate'] < date:
                # check that next step start_date is later than this intervention date
                if not next_step or next_step.get('in_discussion') or next_step['date'] >= date:
                    step['enddate'] = date
                    print('INFO: change end date of', step_i, 'thanks to', interv_file)
                else:
                    # the intervention is for a later step, we find the matching step and move the file
                    for test_step_i in range(step_i, len(steps)):
                        test_step = steps[test_step_i]

                        test_prev_step = list_get_or_none(steps, test_step_i - 1)
                        test_next_step = list_get_or_none(steps, test_step_i + 1)
                        if test_prev_step['stage'] == test_step['stage']:
                            test_prev_step = list_get_or_none(steps, test_step_i - 2)

                        # if the intervention date is between:
                        #    - previous step enddate
                        #    - next step start_date
                        #  => then move the intervention to the current step
                        # + check if we are still in the same chamber
                        if (not test_prev_step or test_prev_step['enddate'] <= date) \
                                and (not test_next_step or test_next_step.get('in_discussion') or date <= test_next_step['date']) \
                                and test_step['institution'] == step['institution']:
                            interventions_files_moved.append(interv_file)
                            print('INFO: moves ', interv_file,' from step', step_i, 'to step', test_step_i)

                            test_interv_dirpath = os.path.join(context.sourcedir, 'procedure', test_step['directory'], 'interventions')
                            if 'intervention_files' not in test_step:
                                test_step['has_interventions'] = True
                                test_step['intervention_files'] = []
                                os.makedirs(test_interv_dirpath)

                            test_step['intervention_files'].append(interv_file)
                            test_step['intervention_files'].sort()
                            os.rename(interv_dirpath + '/' + interv_file + '.json', test_interv_dirpath + '/' + interv_file + '.json')
                            break

        # replace step intervention files
        new_interventions_files = [file for file in step['intervention_files'] if file not in interventions_files_moved]
        step['has_interventions'] = len(new_interventions_files) > 0
        if new_interventions_files:
            step['intervention_files'] = new_interventions_files
        else:
            del step['intervention_files']
            os.rmdir(interv_dirpath)

    return procedure

if __name__ == '__main__':
    process(sys.argv[1], open_json(os.path.join(sys.argv[1], 'viz', 'procedure.json')))



import sys

from lawfactory_utils.urls import download, enable_requests_cache

from .common import download_daily


def process(output_directory):
    for url in "2007-2012.nosdeputes", "2012-2017.nosdeputes", "2017-2022.nosdeputes", "www.nosdeputes", "www.nossenateurs":
        download_daily(
            "https://%s.fr/organismes/groupe/json" % url,
            '%s-groupes' % url,
            output_directory
        )
        download_daily(
            "https://%s.fr/%s/json" %
                (url, 'deputes' if 'deputes' in url else 'senateurs'),
            '%s.parlementaires' % url,
            output_directory
        )
    download_daily(
        'http://data.senat.fr/data/senateurs/ODSEN_HISTOGROUPES.json',
        'historique-groupes-senat',
        output_directory
    )


if __name__ == '__main__':
    enable_requests_cache()
    process(sys.argv[1])



import sys

from anpy.dossier_from_opendata import download_open_data_doslegs

from tlfp.tools.common import download_daily


def process(output_directory):
    all_data = {}
    for legislature in 14, 15:
        all_data[legislature] = download_daily(
            lambda : download_open_data_doslegs(legislature),
            'opendata_AN_dossiers_%d' % legislature,
            output_directory
        )
    return all_data


if __name__ == '__main__':
    process(sys.argv[1])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os, sys, re

from tlfp.tools.common import *

re_short_orga = re.compile(r'(,| et) (à |aux |d).*$')
def clean_orga(orga):
    orga = orga.replace(" de l'assemblée nationale", "")
    orga = orga.replace(" du sénat", "")
    orga = re_short_orga.sub('', orga)
    return orga

def init_section(dic, key, inter, order):
    if inter[key] and inter['seance_lieu'] != 'Hémicycle':
        inter[key] = clean_orga(inter['seance_lieu']) + " <br/> " + inter[key]
    if inter[key] not in dic:
        dic[inter[key]] = {
            'source': inter['source'],
            'link': inter.get('url_nosdeputes', inter.get('url_nossenateurs')),
            'first_date': inter['date'],
            'last_date': inter['date'],
            'total_intervs': 0,
            'total_mots': 0,
            'groupes': {},
            'order': order
        }
        order += 1
    dic[inter[key]]['last_date'] = inter['date']
    return order

def add_intervs(dic, key, inter):
    if key not in dic:
        dic[key] = {
            'nb_intervs': 0,
            'nb_mots': 0
        }
    dic[key]['nb_intervs'] += 1
    dic[key]['nb_mots'] += int(inter['nbmots'])

re_hash = re.compile(r'\W+')
hash_name = lambda x: re_hash.sub('', x).lower()
def save_hash(i, dico, val=None):
    if not val:
        val = i['intervenant_fonction']
    h = hash_name(i['intervenant_nom'])
    if h not in dico or len(dico[h]) < len(val):
        dico[h] = val
def get_hash(i, dico):
    h = hash_name(i['intervenant_nom'])
    return dico[h] if h in dico else ""

re_gouv = re.compile('(ministre|garde.*sceaux|secr[eéÉ]taire.*[eéÉ]tat|haut-commissaire)', re.I)
re_parl = re.compile('(d[eéÉ]put[eéÉ]|s[eéÉ]nateur|membre du parlement|parlementaire)', re.I)
re_rapporteur = re.compile(r'((vice|co|pr[eéÉ]sidente?)[,\-\s]*)?rapporte', re.I)

re_id_laststep = re.compile(r'/[^/\d]*(\d+)\D[^/]*$')

DEBUG = '--debug' in sys.argv

def process(OUTPUT_DIR, procedure):
    context = Context(OUTPUT_DIR, load_parls=True)

    gouv_members = {}
    rapporteurs = {}
    orat_gpes = {}
    def save_gm(i):
        save_hash(i, gouv_members)
    def get_gm(i):
        return get_hash(i, gouv_members)
    def save_rap(i):
        save_hash(i, rapporteurs)
    def get_rap(i):
        return get_hash(i, rapporteurs)
    def save_o_g(i, gpe):
        save_hash(i, orat_gpes, gpe)
    def get_o_g(i):
        return get_hash(i, orat_gpes)

    steps = {}
    id_step = None
    for step in procedure['steps']:
        legislature = step.get('assemblee_legislature', procedure.get('assemblee_legislature'))
        done_links = {}
        id_laststep = id_step
        try:
            id_step = re_id_laststep.search(step["source_url"]).group(1)
        except:
            id_step = None
        if not ('has_interventions' in step and step['has_interventions']):
            continue
        intervs = []
        step['intervention_files'].sort()
        warndone = []
        for seance_file in step['intervention_files']:
            interventions = open_json(os.path.join(context.sourcedir, 'procedure', step['directory'], 'interventions'), "%s.json" % seance_file)['seance']
            has_tag_loi = 0
            if id_laststep:
                for i in interventions:
                    if {"loi": id_laststep} in i['intervention']['lois']:
                        has_tag_loi += 1
            for i in interventions:
                del(i['intervention']['contenu'])
                if has_tag_loi > 2 and {"loi": id_laststep} not in i['intervention']['lois']:
                    if DEBUG:
                        print("SKIPPING interv " + i['intervention']['id'] + " with missing tag loi", file=sys.stderr)
                    continue
                intervs.append(i)

        if step.get('institution') == 'assemblee' and intervs:
            legislature = national_assembly_text_legislature(intervs[0]['intervention']['source'])
        typeparl, urlapi = identify_room(step.get('institution'), legislature)

        # By default divide in subsections, or by seance if no subsection
        sections = {}
        seances = {}
        sec_order = se_order = 1
        for i in intervs:
            i['intervention']['intervenant_fonction'] = i['intervention'].get('intervenant_fonction', '') or ''
            se_order = init_section(seances, 'seance_titre', i['intervention'], se_order)
            sec_order = init_section(sections, 'soussection', i['intervention'], sec_order)
        sectype = 'soussection'
        if len(sections) < 2:
            sectype = 'seance_titre'
            sections = seances

        groupes = {}
        orateurs = {}
        orat_gpes = {}
        for inter in intervs:
            i = inter['intervention']
            if not i['intervenant_nom']:
                continue
            sections[i[sectype]]['total_intervs'] += 1
            sections[i[sectype]]['total_mots'] += int(i['nbmots'])

            # Fix groupes not historicized in NosSénateurs
            if typeparl == "senateur" and i["intervenant_slug"]:
                i["intervenant_groupe"] = context.get_senateur_groupe(i["intervenant_slug"], i["date"], urlapi)

            # Consider as separate groups cases such as: personnalités, présidents and rapporteurs
            gpe = i['intervenant_groupe']
            i['intervenant_fonction'] = decode_html(i['intervenant_fonction'])
            if i['intervenant_fonction'].lower() in ["président", "présidente"]:
                gpe = "Présidence"
            elif re_rapporteur.match(i['intervenant_fonction']):
                gpe = "Rapporteurs"
                save_rap(i)
            elif re_gouv.search(i['intervenant_fonction']):
                gpe = "Gouvernement"
                save_gm(i)
            elif not gpe and re_parl.search(i['intervenant_fonction']+' '+i['intervenant_nom']):
                gpe = "Autres parlementaires"
                # unmeaningful information hard to rematch to the groups, usually invectives, skipping it
                if DEBUG and i['intervenant_nom'] not in warndone:
                    warndone.append(i['intervenant_nom'])
                    print('WARNING: skipping interventions from %s at %s\n' % (i['intervenant_nom'], i['url_nos%ss' % typeparl]), file=sys.stderr)
                continue
            if not gpe:
                if DEBUG and i['intervenant_nom'] not in warndone:
                    warndone.append(i['intervenant_nom'])
                    print('WARNING: neither groupe nor function found for %s at %s\n' % (i['intervenant_nom'], i['url_nos%ss' % typeparl]), file=sys.stderr)
                gm = get_gm(i)
                if gm:
                    gpe = "Gouvernement"
                    i['intervenant_fonction'] = gm
                else:
                    gpe = "Auditionnés"
            else:
                ra = get_rap(i)
                if ra:
                    gpe = "Rapporteurs"
                    i['intervenant_fonction'] = ra
            existing = get_o_g(i)
            if not (existing and gpe == "Présidence") and gpe != get_o_g(i):
                save_o_g(i, gpe)

        for inter in intervs:
            i = inter['intervention']
            gpe = get_o_g(i)
            if not gpe:
                continue
            gpid = context.add_groupe(groupes, get_o_g(i), urlapi)
            add_intervs(sections[i[sectype]]['groupes'], gpid, i)

            # Consider as two separate speakers a same perso with two different fonctions
            orateur = i['intervenant_nom'].lower()
            if orateur not in orateurs:
                orateurs[orateur] = {'nom': i['intervenant_nom'],
                                     'fonction': i['intervenant_fonction'],
                                     'groupe': i['intervenant_groupe'],
                                     'color': '#888888',
                                     'link': parl_link(i, urlapi),
                                     'photo': photo_link(i, urlapi)}
                if i['intervenant_groupe'] and i['intervenant_groupe'].upper() in context.allgroupes[urlapi]:
                    orateurs[orateur]['color'] = context.allgroupes[urlapi][i['intervenant_groupe'].upper()]['color']
            else:
                if len(i['intervenant_fonction']) > len(orateurs[orateur]['fonction']):
                    if DEBUG and ((orateurs[orateur]['fonction'] and not i['intervenant_fonction'].startswith(orateurs[orateur]['fonction'])) or
                      (not orateurs[orateur]['fonction'] and not (i['intervenant_fonction'].startswith('rapporte') or i['intervenant_fonction'].startswith('président')))):
                        print('WARNING: found different functions for %s at %s : %s / %s\n' % (i['intervenant_nom'], i['url_nos%ss' % typeparl], orateurs[orateur]['fonction'], i['intervenant_fonction']), file=sys.stderr)
                    orateurs[orateur]['fonction'] = i['intervenant_fonction']

            if not "orateurs" in sections[i[sectype]]['groupes'][gpid]:
                sections[i[sectype]]['groupes'][gpid]['orateurs'] = {}
            add_intervs(sections[i[sectype]]['groupes'][gpid]['orateurs'], orateur, i)

            orat_sec = "%s-%s-%s" % (i[sectype],gpid,orateur)
            if orat_sec not in done_links:
                if not "link" in sections[i[sectype]]['groupes'][gpid]['orateurs'][orateur] or int(i['nbmots']) > 20:
                    sections[i[sectype]]['groupes'][gpid]['orateurs'][orateur]['link'] = i['url_nos%ss' % typeparl]
                    if int(i['nbmots']) > 20:
                        done_links[orat_sec] = True

        # Remove sections with less than 3 interventions or invalid section name
        for s in dict(sections):
            if not s or sections[s]['total_intervs'] < 3 or sections[s]['total_mots'] < 150 or list(sections[s]['groupes'].keys()) == ['Présidence']:
                del(sections[s])

        if sections:
            steps[step['directory']] = {
                'groupes': groupes,
                'orateurs': orateurs,
                'divisions': sections,
                'total_seances': len(seances),
            }

    print_json(steps, os.path.join(context.sourcedir, 'viz/interventions.json'))


if __name__ == '__main__':
    process(sys.argv[1], json.load(open(os.path.join(sys.argv[1], 'viz/procedure.json'))))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os, sys

from tlfp.tools.common import format_date, datize, print_json, open_json, open_csv

sourcedir = sys.argv[1]
if not sourcedir:
    sys.stderr.write('ERROR: no input directory given\n')
    exit(1)

pagesize = 50
if len(sys.argv) > 2:
    try:
        pagesize = int(sys.argv[2])
    except:
        sys.stderr.write('ERROR: pagesize given as input should be an integer: %s\n' % sys.argv[2])
        exit(1)

dossiers = open_csv(sourcedir, 'dossiers.csv')
total = len(dossiers)


def last_known_activity(d):
    # TODO: use last step date
    return d["Date de promulgation"] or d["Date initiale"]


dossiers.sort(key=lambda k: format_date(last_known_activity(k)), reverse=True)

namefile = lambda npage: "dossiers_%s.json" % npage
def save_json_page(tosave, done):
    npage = (done - 1) // pagesize
    data = {"total": total,
            "count": len(tosave),
            "page": npage,
            "next_page": None,
            "dossiers": tosave}
    if done < total:
        data["next_page"] = namefile(npage+1)
    print('[assemble_procedure] >', namefile(npage))
    print_json(data, os.path.join(sourcedir, namefile(npage)))

done = 0
tosave = []

for d in dossiers:
    proc = open_json(os.path.join(sourcedir, d['id'], 'viz'), 'procedure.json')
    proc["id"] = d["id"]

    for f in ["table_concordance", "objet_du_texte"]:
        if f in proc:
            proc.pop(f)

    tosave.append(proc)
    done += 1
    if done % pagesize == 0:
        save_json_page(tosave, done)
        tosave = []

if tosave:
    save_json_page(tosave, done)



import re
import sys

from lawfactory_utils.urls import download, enable_requests_cache

from .common import strip_text

# TODO:
# - count articles
# - count alineas

re_br = re.compile(r"[\s\n\r]*</?br[^>]*>[\s\n\r]*", re.S)
clean_br = lambda x: re_br.sub("\n", x)
re_fioritures = re.compile(r"(<div[^>]*>[\s\n]*</div>|<a[^>]*>En savoir plus sur [^<]*</a>|Fait à [^,]+, le \d[^.,]*, en double exemplaire\.)")
clean_fioritures = lambda x: re_fioritures.sub("", x)
re_texte = re.compile(r"^.*Le Président (?:de la République (?:française )?)?promulgue la loi dont la teneur suit :(.*?)(La présente loi sera exécutée comme loi de l'Etat\.|<!-- end texte -->|Fait(?: (?:à|au) [^,]+,)? le \d.*?<[^>]*>).*$", re.S|re.I)
extract_contenu = lambda x: re_texte.sub(r"\1", x)
re_reorder = re.compile(r"(Fait(?: (?:à|au) [^,]+,)? le \d.*?<[^>]*>)(.*)(<!-- end texte -->.*Par le Président de la République :)", re.S)
reorder = lambda x: re_reorder.sub(r"\2\1\3", x)
re_signataires = re.compile(r"^.*?((<!-- end texte -->|Fait(?: (?:à|au) [^,]+,)? le \d.*?<[^>]*>).*?)<(font|!-- end signataires).*$", re.S)
signataires = lambda x: re_signataires.sub(r"\1", x)
re_ministre = re.compile(r"le Président de la République|L[ea] (Premi[eè]re?|ministre|garde|secrétaire|haut-commissaire) ")
count_ministres = lambda x: len(re_ministre.findall(x))

def download_texte(url):
    text = download(url).text
    return clean_fioritures(clean_br(text))

def extract_texte(url):
    text = download_texte(url)
    if not re_texte.search(text):
        print("ERROR: could not find texte in JO", url, file=sys.stderr)
        return None
    return strip_text(extract_contenu(text))

def get_texte_length(url):
    text = extract_texte(url)
    if not text:
        return -1
    return len(text)

def extract_signataires(url):
    text_src = reorder(download_texte(url))
    if not re_signataires.search(text_src):
        print("ERROR: could not find signature in texte JO", url, file=sys.stderr)
        return None
    text_src = signataires(text_src)
    return strip_text(text_src)

def count_signataires(url):
    signataires_text = extract_signataires(url)
    if not signataires_text:
        return -1
    return count_ministres(signataires_text)

if __name__ == "__main__":
    enable_requests_cache()
    if len(sys.argv) == 2:
        with open(sys.argv[1]) as f:
            for url in f.readlines():
                url = url.strip()
                print(url, ':', get_texte_length(url), '|', count_signataires(url))
    else:
        print(extract_texte(sys.argv[1]), get_texte_length(sys.argv[1]))
        print(extract_signataires(sys.argv[1]), count_signataires(sys.argv[1]))



# -*- coding: utf-8 -*-
from setuptools import find_packages, setup
from codecs import open
from os import path
import re

here = path.abspath(path.dirname(__file__))

__version__ = '0.0.3'

with open(path.join(here, 'README.md')) as readme:
    LONG_DESC = readme.read()

with open(path.join(here, 'requirements.txt')) as f:
    requirements = [line.strip() for line in f.read().splitlines()]

setup(
    name='tlfp',
    version=__version__,

    description='Data generator for the-law-factory project lafabriquedelaloi.fr',
    long_description=LONG_DESC,
    long_description_content_type='text/markdown',
    license="GPL",

    url='https://github.com/regardscitoyens/the-law-factory-parser',
    author='Regards Citoyens',
    author_email='contact@regardscitoyens.org',
    include_package_data=True,

    classifiers=[
        'Development Status :: 1 - Planning',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],

    keywords='scraping politics data',

    packages=find_packages(),

    install_requires=requirements,

    scripts=[
        'bin/tlfp-parse-text',
        'bin/tlfp-parse',
        'bin/tlfp-parse-many',
    ],
)



"""
Debugging tool to compare between the old parser and the new one
"""

import sys

from .common import open_json


def compare(proc, me, verbose=True):
    score_ok = 0
    score_nok = 0

    myprint = print
    if not verbose:
        myprint = lambda *args: None

    # i like to write cryptic function sometimes also
    def test_test(proc, me):
        def test(a, b=None, a_key=lambda a: a, ok_if_correct_is_none=False):
            nonlocal score_nok, score_ok

            def clean(obj):
                if type(obj) is str:
                    # http == https
                    obj = obj.replace('https://', 'http://')
                    obj = obj.replace('/www.legifrance.gouv.fr', '/legifrance.gouv.fr')
                    # old senat url
                    obj = obj.replace('/dossierleg/', '/dossier-legislatif/')
                    obj = obj.replace('&categorieLien=id', '')
                    obj = obj.replace('/./', '/')
                    # 1ère lecture VS 1ere lecture, should be standardized
                    return obj.replace('è', 'e')
                return obj

            if b is None:
                b = a
            a_val = clean(a_key(proc.get(a)))
            b_val = clean(me.get(b))
            if a_val != b_val and not (not a_val and not b_val) and not (not a_val and ok_if_correct_is_none):
                # rapport/ta-commission can be guessed
                if not (type(a_val) is str and type(b_val) is str and \
                    (a_val.replace('/rapports/', '/ta-commission/') \
                        == b_val.replace('/rapports/', '/ta-commission/')
                    )):
                    print('!! NOK !!', a,' diff:', a_val, 'VS', b_val)
                    score_nok += 1
                    return
            print('OK', a, '(', a_val, ')')
            score_ok += 1
        return test

    test = test_test(proc, me)
    #test('beginning')
    #test('short_title', a_key=lambda x: x.replace('(texte organique)','').strip())
    #test('long_title', 'long_title_descr')
    #test('end')
    test('url_dossier_assemblee')
    test('url_dossier_senat')
    test('url_jo', ok_if_correct_is_none=True)
    test('type', 'urgence', lambda type: type == 'urgence')

    myprint()
    myprint('STEPS:')
    if len(proc['steps']) != len(me['steps']):
        myprint('!! NOK !! DIFFERENT NUMBER OF STEPS:', len(proc['steps']), 'VS', len(me['steps']))
    myprint()

    for i, step_proc in enumerate(proc['steps']):
        myprint(' - step', i + 1)

        step_me = {}
        if len(me['steps']) > i:
            step_me = me['steps'][i]
        else:
            myprint('  - step not in mine')

        test = test_test(step_proc, step_me)

        # test('date')
        test('institution')
        test('stage')
        test('step')
        test('source_url')
        myprint()

    myprint('NOK:', score_nok)
    myprint('OK:', score_ok)
    return score_nok, score_ok

if __name__ == '__main__':
    import glob

    sum_ok = 0
    sum_nok = 0
    missing = 0
    perfect = 0
    less_than_1 = 0

    all_doslegs = open_json(sys.argv[2])
    lafabrique_doslegs = list(sorted(glob.glob(sys.argv[1])))
    scored = []
    for file in lafabrique_doslegs:
        print('======')
        print('======')
        print(file)
        me = None
        proc = open_json(file)
        proc_url_senat = proc.get('url_dossier_senat', '').replace('http://', 'https://').replace('/dossierleg/', '/dossier-legislatif/')
        for dos in all_doslegs:
            dos_url_senat = dos.get('url_dossier_senat', '').replace('http://', 'https://').replace('/dossierleg/', '/dossier-legislatif/')
            if dos_url_senat == proc_url_senat:
                me = dos
                break
        # look at a common url for AN after senat since the senat individualize their doslegs
        if not me:
            for dos in all_doslegs:
                if dos.get('url_dossier_assemblee') == proc.get('url_dossier_assemblee'):
                    me = dos
                    break
        if not me:
            missing += 1
            print(file)
            print('NO DOSLEGS FOUND:', proc['url_dossier_senat'])
            continue
        nok, ok = compare(proc, me)
        sum_ok += ok
        sum_nok += nok
        scored.append([file, nok, ok])

        if nok == 0:
            perfect += 1
        if nok <= 1:
            less_than_1 += 1

    print('-----')
    print('TOTAL:')
    print('MISSING DOSLEGS', missing, '/', len(lafabrique_doslegs))
    print('** PERFECT **', perfect, '/', len(lafabrique_doslegs))
    print('** NOK <= 1 **', less_than_1, '/', len(lafabrique_doslegs))
    print('NOK',sum_nok)
    print('OK', sum_ok)
    print('=>', (sum_ok/(sum_nok+sum_ok))*100)
    print()
    print('worst:')
    for file, nok, ok in sorted(scored, key=lambda x:x[1])[-5:]:
        print(file, nok, ok)


# missing steps
# /pjl09-113

# double cmp hemi
# pjl10-784
# pjl11-187 (ordering wtf)

# NOK 11
# ppl08-454
# ppl09-191
